@@ -24,24 +24,24 @@
 #include <unistd.h>
 #include <string.h>
 #ifdef ENABLE_HMAC_BINARY_CHECK
-# include <dlfcn.h> 
+# include <dlfcn.h>
 #endif
 #ifdef HAVE_SYSLOG
 # include <syslog.h>
 #endif /*HAVE_SYSLOG*/
 
 #include "g10lib.h"
-#include "ath.h"
 #include "cipher-proto.h"
 #include "hmac256.h"
 
 
-/* The name of the file used to foce libgcrypt into fips mode. */
-#define FIPS_FORCE_FILE "/etc/gcrypt/fips_enabled"
+/* The name of the file used to force libgcrypt into fips mode. */
+/* Note: Always use get_fips_force_file to get this name.       */
+#define FIPS_FORCE_FILE "fips_enabled"
 
 
 /* The states of the finite state machine used in fips mode.  */
-enum module_states 
+enum module_states
   {
     /* POWEROFF cannot be represented.  */
     STATE_POWERON  = 0,
@@ -55,7 +55,7 @@ enum module_states
 
 
 /* Flag telling whether we are in fips mode.  It uses inverse logic so
-   that fips mode is the default unless changed by the intialization
+   that fips mode is the default unless changed by the initialization
    code. To check whether fips mode is enabled, use the function
    fips_mode()! */
 static int no_fips_mode_required;
@@ -69,7 +69,7 @@ static int enforced_fips_mode;
 static int inactive_fips_mode;
 
 /* This is the lock we use to protect the FSM.  */
-static ath_mutex_t fsm_lock = ATH_MUTEX_INITIALIZER;
+GPGRT_LOCK_DEFINE (fsm_lock);
 
 /* The current state of the FSM.  The whole state machinery is only
    used while in fips mode. Change this only while holding fsm_lock. */
@@ -92,6 +92,27 @@ static void fips_new_state (enum module_states new_state);
 
 
 
+static const char *
+get_fips_force_file (void)
+{
+#ifdef HAVE_W32_SYSTEM
+  static char *fname;
+
+  if (!fname)
+    {
+      const char *sysconfdir = _gcry_get_sysconfdir();
+
+      fname = xmalloc (strlen (sysconfdir) + strlen (FIPS_FORCE_FILE) + 1);
+      strcpy (fname, sysconfdir);
+      strcat (fname, FIPS_FORCE_FILE);
+    }
+  return fname;
+#else
+  return "/etc/gcrypt/" FIPS_FORCE_FILE;
+#endif
+}
+
+
 /* Check whether the OS is in FIPS mode and record that in a module
    local variable.  If FORCE is passed as true, fips mode will be
    enabled anyway. Note: This function is not thread-safe and should
@@ -102,8 +123,8 @@ _gcry_initialize_fips_mode (int force)
 {
   static int done;
   gpg_error_t err;
-      
-  /* Make sure we are not accidently called twice.  */
+
+  /* Make sure we are not accidentally called twice.  */
   if (done)
     {
       if ( fips_mode () )
@@ -128,13 +149,14 @@ _gcry_initialize_fips_mode (int force)
      file.  The filename is hardwired so that there won't be any
      confusion on whether /etc/gcrypt/ or /usr/local/etc/gcrypt/ is
      actually used.  The file itself may be empty.  */
-  if ( !access (FIPS_FORCE_FILE, F_OK) )
+  if ( !access (get_fips_force_file (), F_OK) )
     {
       gcry_assert (!no_fips_mode_required);
       goto leave;
     }
 
   /* Checking based on /proc file properties.  */
+#ifndef HAVE_W32_SYSTEM
   {
     static const char procfname[] = "/proc/sys/crypto/fips_enabled";
     FILE *fp;
@@ -144,7 +166,7 @@ _gcry_initialize_fips_mode (int force)
     if (fp)
       {
         char line[256];
-        
+
         if (fgets (line, sizeof line, fp) && atoi (line))
           {
             /* System is in fips mode.  */
@@ -170,7 +192,8 @@ _gcry_initialize_fips_mode (int force)
         abort ();
       }
   }
-  
+#endif /*!HAVE_W32_SYSTEM*/
+
   /* Fips not not requested, set flag.  */
   no_fips_mode_required = 1;
 
@@ -181,30 +204,30 @@ _gcry_initialize_fips_mode (int force)
       FILE *fp;
 
       /* Intitialize the lock to protect the FSM.  */
-      err = ath_mutex_init (&fsm_lock);
+      err = gpgrt_lock_init (&fsm_lock);
       if (err)
         {
           /* If that fails we can't do anything but abort the
              process. We need to use log_info so that the FSM won't
              get involved.  */
           log_info ("FATAL: failed to create the FSM lock in libgcrypt: %s\n",
-                     strerror (err));
+                    gpg_strerror (err));
 #ifdef HAVE_SYSLOG
           syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
                   "creating FSM lock failed: %s - abort",
-                  strerror (err));
+                  gpg_strerror (err));
 #endif /*HAVE_SYSLOG*/
           abort ();
         }
 
-      
+
       /* If the FIPS force files exists, is readable and has a number
          != 0 on its first line, we enable the enforced fips mode.  */
-      fp = fopen (FIPS_FORCE_FILE, "r");
+      fp = fopen (get_fips_force_file (), "r");
       if (fp)
         {
           char line[256];
-          
+
           if (fgets (line, sizeof line, fp) && atoi (line))
             enforced_fips_mode = 1;
           fclose (fp);
@@ -212,7 +235,7 @@ _gcry_initialize_fips_mode (int force)
 
       /* Now get us into the INIT state.  */
       fips_new_state (STATE_INIT);
-      
+
     }
   return;
 }
@@ -222,15 +245,15 @@ lock_fsm (void)
 {
   gpg_error_t err;
 
-  err = ath_mutex_lock (&fsm_lock);
+  err = gpgrt_lock_lock (&fsm_lock);
   if (err)
     {
-      log_info ("FATAL: failed to acquire the FSM lock in libgrypt: %s\n", 
-                strerror (err));
+      log_info ("FATAL: failed to acquire the FSM lock in libgrypt: %s\n",
+                gpg_strerror (err));
 #ifdef HAVE_SYSLOG
       syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
               "acquiring FSM lock failed: %s - abort",
-              strerror (err));
+              gpg_strerror (err));
 #endif /*HAVE_SYSLOG*/
       abort ();
     }
@@ -241,15 +264,15 @@ unlock_fsm (void)
 {
   gpg_error_t err;
 
-  err = ath_mutex_unlock (&fsm_lock);
+  err = gpgrt_lock_unlock (&fsm_lock);
   if (err)
     {
       log_info ("FATAL: failed to release the FSM lock in libgrypt: %s\n",
-                strerror (err));
+                gpg_strerror (err));
 #ifdef HAVE_SYSLOG
       syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
               "releasing FSM lock failed: %s - abort",
-              strerror (err));
+              gpg_strerror (err));
 #endif /*HAVE_SYSLOG*/
       abort ();
     }
@@ -271,12 +294,20 @@ _gcry_fips_mode (void)
 
 
 /* Return a flag telling whether we are in the enforced fips mode.  */
-int 
+int
 _gcry_enforced_fips_mode (void)
 {
+  if (!_gcry_fips_mode ())
+    return 0;
   return enforced_fips_mode;
 }
 
+/* Set a flag telling whether we are in the enforced fips mode.  */
+void
+_gcry_set_enforced_fips_mode (void)
+{
+  enforced_fips_mode = 1;
+}
 
 /* If we do not want to enforce the fips mode, we can set a flag so
    that the application may check whether it is still in fips mode.
@@ -347,7 +378,7 @@ state2str (enum module_states state)
 
 
 /* Return true if the library is in the operational state.  */
-int 
+int
 _gcry_fips_is_operational (void)
 {
   int result;
@@ -370,7 +401,7 @@ _gcry_fips_is_operational (void)
              (GCRYCTL_INITIALIZATION_FINISHED) where the latter will
              run the selftests.  The drawback of these on-demand
              self-tests are a small chance that self-tests are
-             performed by severeal threads; that is no problem because
+             performed by several threads; that is no problem because
              our FSM make sure that we won't oversee any error. */
           unlock_fsm ();
           _gcry_fips_run_selftests (0);
@@ -384,7 +415,7 @@ _gcry_fips_is_operational (void)
 }
 
 
-/* This is test on wether the library is in the operational state.  In
+/* This is test on whether the library is in the operational state.  In
    contrast to _gcry_fips_is_operational this function won't do a
    state transition on the fly.  */
 int
@@ -437,8 +468,8 @@ reporter (const char *domain, int algo, const char *what, const char *errtxt)
             !strcmp (domain, "digest")? _gcry_md_algo_name (algo) :
             !strcmp (domain, "hmac")?   _gcry_md_algo_name (algo) :
             !strcmp (domain, "pubkey")? _gcry_pk_algo_name (algo) : "",
-            algo, errtxt? errtxt:"Okay", 
-            what?" (":"", what? what:"", what?")":""); 
+            algo, errtxt? errtxt:"Okay",
+            what?" (":"", what? what:"", what?")":"");
 }
 
 /* Run self-tests for all required cipher algorithms.  Return 0 on
@@ -446,7 +477,7 @@ reporter (const char *domain, int algo, const char *what, const char *errtxt)
 static int
 run_cipher_selftests (int extended)
 {
-  static int algos[] = 
+  static int algos[] =
     {
       GCRY_CIPHER_3DES,
       GCRY_CIPHER_AES128,
@@ -475,7 +506,7 @@ run_cipher_selftests (int extended)
 static int
 run_digest_selftests (int extended)
 {
-  static int algos[] = 
+  static int algos[] =
     {
       GCRY_MD_SHA1,
       GCRY_MD_SHA224,
@@ -504,13 +535,17 @@ run_digest_selftests (int extended)
 static int
 run_hmac_selftests (int extended)
 {
-  static int algos[] = 
+  static int algos[] =
     {
       GCRY_MD_SHA1,
       GCRY_MD_SHA224,
       GCRY_MD_SHA256,
       GCRY_MD_SHA384,
       GCRY_MD_SHA512,
+      GCRY_MD_SHA3_224,
+      GCRY_MD_SHA3_256,
+      GCRY_MD_SHA3_384,
+      GCRY_MD_SHA3_512,
       0
     };
   int idx;
@@ -534,11 +569,11 @@ run_hmac_selftests (int extended)
 static int
 run_pubkey_selftests (int extended)
 {
-  static int algos[] = 
+  static int algos[] =
     {
       GCRY_PK_RSA,
       GCRY_PK_DSA,
-      /* GCRY_PK_ECDSA is not enabled in fips mode.  */
+      GCRY_PK_ECC,
       0
     };
   int idx;
@@ -566,7 +601,7 @@ run_random_selftests (void)
 
   err = _gcry_random_selftest (reporter);
   reporter ("random", 0, NULL, err? gpg_strerror (err):NULL);
-  
+
   return !!err;
 }
 
@@ -581,7 +616,7 @@ check_binary_integrity (void)
   int dlen;
   char *fname = NULL;
   const char key[] = "What am I, a doctor or a moonshuttle conductor?";
-  
+
   if (!dladdr ("gcry_check_version", &info))
     err = gpg_error_from_syserror ();
   else
@@ -594,7 +629,7 @@ check_binary_integrity (void)
         err = gpg_error (GPG_ERR_INTERNAL);
       else
         {
-          fname = gcry_malloc (strlen (info.dli_fname) + 1 + 5 + 1 );
+          fname = xtrymalloc (strlen (info.dli_fname) + 1 + 5 + 1 );
           if (!fname)
             err = gpg_error_from_syserror ();
           else
@@ -621,16 +656,20 @@ check_binary_integrity (void)
                 {
                   /* A buffer of 64 bytes plus one for a LF and one to
                      detect garbage.  */
-                  unsigned char buffer[64+1+1]; 
+                  unsigned char buffer[64+1+1];
                   const unsigned char *s;
                   int n;
 
                   /* The HMAC files consists of lowercase hex digits
-                     only with an optional trailing linefeed.  Fail if
-                     there is any garbage.  */
+                     with an optional trailing linefeed or optional
+                     with two trailing spaces.  The latter format
+                     allows the use of the usual sha1sum format.  Fail
+                     if there is any garbage.  */
                   err = gpg_error (GPG_ERR_SELFTEST_FAILED);
                   n = fread (buffer, 1, sizeof buffer, fp);
-                  if (n == 64 || (n == 65 && buffer[64] == '\n'))
+                  if (n == 64
+                      || (n == 65 && buffer[64] == '\n')
+                      || (n == 66 && buffer[64] == ' ' && buffer[65] == ' '))
                     {
                       buffer[64] = 0;
                       for (n=0, s= buffer;
@@ -652,7 +691,7 @@ check_binary_integrity (void)
             "integrity check using `%s' failed: %s",
             fname? fname:"[?]", gpg_strerror (err));
 #endif /*HAVE_SYSLOG*/
-  gcry_free (fname);
+  xfree (fname);
   return !!err;
 #else
   return 0;
@@ -667,7 +706,7 @@ _gcry_fips_run_selftests (int extended)
 {
   enum module_states result = STATE_ERROR;
   gcry_err_code_t ec = GPG_ERR_SELFTEST_FAILED;
-  
+
   if (fips_mode ())
     fips_new_state (STATE_SELFTEST);
 
@@ -726,14 +765,14 @@ _gcry_fips_signal_error (const char *srcfile, int srcline, const char *srcfunc,
   /* Print error.  */
   log_info ("%serror in libgcrypt, file %s, line %d%s%s: %s\n",
             is_fatal? "fatal ":"",
-            srcfile, srcline, 
+            srcfile, srcline,
             srcfunc? ", function ":"", srcfunc? srcfunc:"",
             description? description : "no description available");
 #ifdef HAVE_SYSLOG
   syslog (LOG_USER|LOG_ERR, "Libgcrypt error: "
           "%serror in file %s, line %d%s%s: %s",
           is_fatal? "fatal ":"",
-          srcfile, srcline, 
+          srcfile, srcline,
           srcfunc? ", function ":"", srcfunc? srcfunc:"",
           description? description : "no description available");
 #endif /*HAVE_SYSLOG*/
@@ -766,22 +805,22 @@ fips_new_state (enum module_states new_state)
           || new_state == STATE_FATALERROR)
         ok = 1;
       break;
-      
+
     case STATE_SELFTEST:
       if (new_state == STATE_OPERATIONAL
           || new_state == STATE_ERROR
           || new_state == STATE_FATALERROR)
         ok = 1;
       break;
-      
+
     case STATE_OPERATIONAL:
-      if (new_state == STATE_SHUTDOWN 
+      if (new_state == STATE_SHUTDOWN
           || new_state == STATE_SELFTEST
           || new_state == STATE_ERROR
           || new_state == STATE_FATALERROR)
         ok = 1;
       break;
-      
+
     case STATE_ERROR:
       if (new_state == STATE_SHUTDOWN
           || new_state == STATE_ERROR
@@ -789,18 +828,18 @@ fips_new_state (enum module_states new_state)
           || new_state == STATE_SELFTEST)
         ok = 1;
       break;
-      
+
     case STATE_FATALERROR:
       if (new_state == STATE_SHUTDOWN )
         ok = 1;
       break;
-      
+
     case STATE_SHUTDOWN:
       /* We won't see any transition *from* Shutdown because the only
          allowed new state is Power-Off and that one can't be
          represented.  */
       break;
-      
+
     }
 
   if (ok)
@@ -814,12 +853,12 @@ fips_new_state (enum module_states new_state)
     log_info ("libgcrypt state transition %s => %s %s\n",
               state2str (last_state), state2str (new_state),
               ok? "granted":"denied");
-  
+
   if (!ok)
     {
       /* Invalid state transition.  Halting library. */
 #ifdef HAVE_SYSLOG
-      syslog (LOG_USER|LOG_ERR, 
+      syslog (LOG_USER|LOG_ERR,
               "Libgcrypt error: invalid state transition %s => %s",
               state2str (last_state), state2str (new_state));
 #endif /*HAVE_SYSLOG*/
@@ -828,7 +867,7 @@ fips_new_state (enum module_states new_state)
   else if (new_state == STATE_ERROR || new_state == STATE_FATALERROR)
     {
 #ifdef HAVE_SYSLOG
-      syslog (LOG_USER|LOG_WARNING, 
+      syslog (LOG_USER|LOG_WARNING,
               "Libgcrypt notice: state transition %s => %s",
               state2str (last_state), state2str (new_state));
 #endif /*HAVE_SYSLOG*/
